﻿-------------------------------------------------------------------------------
-- These functions provide integration with and management of Recount which
-- provides for damage and healing meters, etc.
-------------------------------------------------------------------------------

-- local cache

local CreateFrame      = CreateFrame;
local InCombatLockdown = InCombatLockdown;

-------------------------------------------------------------------------------

function nUI:initRecount()

	if nUI.AddOn.recount then
		
		local frame = Recount.MainWindow;
			
		nUI:addInfoMode( "Stats", 
			function() 
				frame:Show(); 
			end, 
			function() 
				local mode = nUI.info_frame.modeName[nUI_Config.activeInfoMode];
				
				if mode ~= "Stats" and mode ~= "KTM" and mode ~= "Omen2" then
					frame:Hide(); 
				elseif InCombatLockdown() then
					frame:Hide();
				else
					frame:Show();
				end 
			end
		);
		
		frame:SetParent( nUI.info_frame.stats );
		frame:SetPoint( "TOPLEFT", nUI.info_frame.stats, "TOPLEFT", 0, 11 );
		frame:SetPoint( "BOTTOMRIGHT", nUI.info_frame.stats, "BOTTOMRIGHT", -5, 0 );
		frame:SetWidth( frame:GetRight() - frame:GetLeft() ); 
		frame:SetFrameStrata( nUI.info_frame.stats:GetFrameStrata() );
		frame:SetFrameLevel( nUI.info_frame.stats:GetFrameLevel()+1 );
		frame:SetHeight( frame:GetTop() - frame:GetBottom() ); 
		frame:SetBackdropBorderColor( 0, 0, 0, 1 );
		frame:SetBackdropColor( 0, 0, 0, 0 );
		frame:Show();
		
		frame.CloseButton:Hide();
		
		Recount:ResizeMainWindow();
		Recount:LockWindows( true );
		
		local event_frame = CreateFrame( "Frame", "$parent_RecountEvents", nUI.info_frame );
		nUI.info_frame.recount_events = event_frame;
		
		event_frame:SetScript( "OnEvent", function() nUI:recountEvent(); end );
		
		event_frame:RegisterEvent( "PLAYER_REGEN_ENABLED" );
		event_frame:RegisterEvent( "PLAYER_REGEN_DISABLED" );
		event_frame:RegisterEvent( "PET_ATTACK_START" );
		event_frame:RegisterEvent( "PET_ATTACK_STOP" );
	end
end

-------------------------------------------------------------------------------

function nUI:recountEvent()
	
	local mode  = nUI.info_frame.modeName[nUI_Config.activeInfoMode];
	local frame = Recount.MainWindow;
	
	if event == "PLAYER_REGEN_DISABLED" or event == "PET_ATTACK_START" then
		if frame:IsShown() and mode ~= "Stats" then
			frame:Hide();
		end
	elseif event == "PLAYER_REGEN_ENABLED" or (event == "PET_ATTACK_STOP" and not InCombatLockdown()) then
		if not frame:IsShown() and (mode == "Stats" or mode == "Omen2" or mode == "KTM") then
			frame:Show();
		end
	end
end
