﻿-------------------------------------------------------------------------------
-- local cache

local CreateFrame = CreateFrame;

-------------------------------------------------------------------------------

nUI_ButtonBag = 
{	
	Excludes = 
	{		
		["MinimapBackdrop"] = 1,
		["MiniMapMailFrame"] = 1,
		["MiniMapBattlefieldFrame"] = 1,
		["MiniMapPing"] = 1,
		["MiniMapCompassRing"] = 1,
		["MiniMapMeetingStoneFrame"] = 1,
		["MiniMapVoiceChatFrame"] = 1,
		["MinimapZoomIn"] = 1,
		["MinimapZoomOut"] = 1,
		["MiniMapTracking"] = 1,
		["MiniMapWorldMapButton"] = 1,
		["MiniMapRecordingButton"] = 1,
		["nUI_PlayerCoords"] = 1,
		["nUI_Dashboard_LocBackdrop"] = 1,
		["GatherMiniNoteUpdateFrame"] = 1,
	},	
	
	Includes = 
	{		
		["WIM_IconFrame"] = 1,
		["CTMod2_MinimapButton"] = 1,
		["PoisonerMinimapButton"] = 1,
		["GameTimeFrame"] = 1,
	},
}

-------------------------------------------------------------------------------

function nUI:buttonBagFix()

	-- create a button container
	
	if not nUI.info_frame.buttonbag then
		
		nUI.info_frame.buttonbag = CreateFrame( "Frame", "$parent_ButtonBag", nUI.info_frame );
		nUI.info_frame.buttonbag:SetHeight( nUI.info_frame:GetHeight()-2 );
		nUI.info_frame.buttonbag:SetWidth( nUI.info_frame:GetWidth() - _G["BattlefieldMinimap1"]:GetWidth()*BattlefieldMinimap:GetScale()*4 );
		nUI.info_frame.buttonbag:SetPoint( "RIGHT", nUI.info_frame, "RIGHT", 0, -1 );

		nUI.info_frame.buttonbag.buttons = {};
	end
	
	-- special processing... the GroupCalendar mod hijacks the minimap time of day
	-- indicator which we normally hide... so... if GroupCalendar is loaded, then we
	-- need to unhide that button and move it.
	
	if IsAddOnLoaded( "GroupCalendar" ) and not GameTimeFrame:IsShown() then
		GameTimeFrame:Show();
	end
	
	-- build a list of children in the minimap
	
	local children    = { Minimap:GetChildren() };
	local children2   = { MinimapBackdrop:GetChildren() };
	local new_buttons = {};

	for _, child in ipairs( children2 ) do 
		table.insert(children, child); 
	end

	-- pick up any custom includes or excludes the player may have
	-- and add them to the default list
	
	nUI.ButtonList = {};
	
	for _,item in ipairs( nUI_Config.ButtonBag.Excludes ) do
		table.insert( nUI_ButtonBag.Excludes, item );
	end

	for _,item in ipairs( nUI_Config.ButtonBag.Includes ) do
		table.insert( nUI_ButtonBag.Includes, item );
	end
	
	-- check the buttons we are forcing a move for
	
	for i in pairs( nUI_ButtonBag.Includes ) do
		
		local move_button = nUI:testMinimapButton( _G[i] );
		
		if move_button and move_button:IsShown() then

			local button_name = move_button:GetName();

			if not nUI.ButtonBag[button_name] then
			
				-- make sure we don't pick this same button up in another scan
				
				move_button:SetParent( nUI.info_frame.buttonbag );
				
				-- moving a button we've seen before			
				
				if nUI_Config.ButtonBag[button_name] then
	
					nUI:addButtonBagButton( {
						name  = button_name,
						row   = nUI_Config.ButtonBag.Buttons[button_name].row,
						col   = nUI_Config.ButtonBag.Buttons[button_name].col,
						frame = move_button,
					} );
					
					nUI.ButtonBag[button_name] = true;
					
				-- moving a new button
				
				else
	
					new_buttons[button_name] = 
					{
						name  = button_name,
						row   = nil,
						col   = nil,
						frame = move_button,
					}				
					
				end
			end
		end
	end
	
	-- parse the list of minimap children and see who might be a button
	-- so we can prepare to move it
	
	for _, child in ipairs( children ) do
		
		local move_button = nUI:testMinimapButton( child );

		if move_button and move_button:IsShown() then
		
			local button_name = move_button:GetName();
	
			if not nUI.ButtonBag[button_name] then
					
				-- make sure we don't pick this same button up in another scan
				
				move_button:SetParent( nUI.info_frame.buttonbag );
				
				-- moving a button we moved before
	
				if nUI_Config.ButtonBag.Buttons[button_name] then
					
					nUI:addButtonBagButton( {
						name  = button_name,
						row   = nUI_Config.ButtonBag.Buttons[button_name].row,
						col   = nUI_Config.ButtonBag.Buttons[button_name].col,
						frame = move_button,
					} );
	
					nUI.ButtonBag[button_name] = true;
								
				-- moving a new button
				
				else
					
					new_buttons[button_name] = 
					{
						name  = button_name,
						row   = nil,
						col   = nil,
						frame = move_button,
					}				
					
				end
			end
		end
	end

	-- now add all of the unknown buttons in the button bag
	
	for i in pairs( new_buttons ) do
	
		nUI:addButtonBagButton( new_buttons[i] );
		nUI.ButtonBag[i] = true;
		
	end
	
	-- now compress the button bag and lay out the buttons
	
	nUI:compressButtonBag();
	
	local bag = nUI.info_frame.buttonbag.buttons;
	local button_size = nUI.info_frame.buttonbag:GetWidth() / 3 - 2;
	
	nUI.info_frame.buttonbag.containers = {};
	
	for i in pairs( bag ) do
		
		nUI.info_frame.buttonbag.containers[i] = {};
		
		for j in pairs( bag[i] ) do
			
			local button    = bag[i][j];
			local scale     = button_size / button.frame:GetWidth();
			local container = CreateFrame( "Button", "$parent_Button_"..i.."_"..j, nUI.info_frame.buttonbag );			

			nUI.info_frame.buttonbag.containers[i][j] = container;
			
			container:SetWidth( button_size );
			container:SetHeight( button_size );
			
			if i == 1 and j == 1 then
				container:SetPoint( "TOPLEFT", nUI.info_frame.buttonbag, "TOPLEFT", 0, 0 );
			elseif j == 1 then
				container:SetPoint( "TOPLEFT", nUI.info_frame.buttonbag.containers[i-1][1], "BOTTOMLEFT", 0, 0 );
			else
				container:SetPoint( "LEFT", nUI.info_frame.buttonbag.containers[i][j-1], "RIGHT", 0, 0 );
			end
			
			button.frame:SetScale( scale );
			button.frame:SetParent( container );
			button.frame:ClearAllPoints();
			button.frame:SetPoint( "CENTER", container, "CENTER", 0, 0 );
			button.frame:RegisterForDrag();
			
		end
	end
end		

-------------------------------------------------------------------------------

function nUI:testMinimapButton( button )

	local result = nil;
	
	if button then
			
		local button_name  = button:GetName();	
		local hasClick     = false;
		local hasMouseUp   = false;
		local hasMouseDown = false;
		
		-- test the current button to see if it is one we can/should/will move
		
		if button_name then

			-- the Gatherer Mod minimap notes are misinterpreted as minimap
			-- buttons. This forces us to ignore all Gatherer note nodes
			
			if button_name:match( "GatherNote." ) then
				return nil;
			end
			
			-- otherwise, determine what to do with the button
			
			if nUI_ButtonBag.Excludes[button_name] then
				
				nUI_Config.ButtonBag.Logging[button_name] = "skipped forced ignore button";
				return nil;
				
			elseif nUI_ButtonBag.Includes[button_name] then
				
				nUI_Config.ButtonBag.Logging[button_name] = "moved forced include button";
				return button;
				
			elseif nUI_Config.ButtonBag.Buttons[button_name] then
				
				nUI_Config.ButtonBag.Logging[button_name] = "moved previously known button";
				return button;
				
			elseif button.HasScript then
	
				if button:HasScript( "OnClick" ) and button:GetScript( "OnClick" ) then 
					hasClick = true;
				end
				
				if button:HasScript( "OnMouseUp" ) and button:GetScript( "OnMouseUp" ) then
					hasMouseUp = true;
				end
				
				if button:HasScript( "OnMouseDown" ) and button:GetScript( "OnMouseDown" ) then
					hasMouseDown = true; 
				end
	
				if hasClick or hasMouseUp or hasMouseDown then
				
					nUI_Config.ButtonBag.Logging[button_name] = "autodetected and moved new button";
					return button;
					
				else
							
					nUI_Config.ButtonBag.Logging[button_name] = "does not appear to be a button";
					
				end				
				
			else				
				nUI_Config.ButtonBag.Logging[button_name] = "does not support \"HasScript()\"";
			end					
		end
		
		-- if we didn't find a button, try diving the children of this frame
		-- recursion ftw
		
		if not result then
			for _,child in ipairs( { button:GetChildren() } ) do
				
				result = nUI:testMinimapButton( child );
				
				if result then 
					child.is_embedded = true;
					break; 
				end
			end
		end
	end
	
	return result;
end

-------------------------------------------------------------------------------

function nUI:buttonBagInsert( bag, button, row, col )

	button.row = row;
	button.col = col;
	
	if not bag[row] then bag[row] = {}; end
	
	bag[row][col] = button;
	
	if not nUI.info_frame.buttonbag.last_row 
	or row > nUI.info_frame.buttonbag.last_row then
		nUI.info_frame.buttonbag.last_row = row;
	end
	
end

-------------------------------------------------------------------------------
-- this routine adds a button to the button bag. If the new button does not
-- have a row or a column, then we look for an empty spot in the table to
-- locate it

function nUI:addButtonBagButton( new_button )

	local row = new_button.row;
	local col = new_button.col;
	local bag = nUI.info_frame.buttonbag.buttons;

	-- make sure the column is a valid number
	
	if col and (col < 1 or col > 3) then col = nil; end
	
	-- if the button has a row and a column then see if that spot is
	-- still open in the button bag. If not.. then bump the button
	
	if row and col then
		
		if not bag[row] then 
			nUI:buttonBagInsert( bag, new_button, row, col );
		elseif not bag[row][col] then 
			nUI:buttonBagInsert( bag, new_button, row, col );
		elseif bag[row][col].name == new_button.name then
			nUI:buttonBagInsert( bag, new_button, row, col );
		else
			row = nil;
		end		
	end

	-- if the new button has not been placed, then look for
	-- the first unused spot in the bag to place it in
	
	if not row or not col then
		
		local new_row = 1;
		local new_col = 1;
		
		while bag[new_row] do
						
			while bag[new_row][new_col] and new_col < 4 do
				new_col = new_col+1;
			end

			if new_col ~= 4 then
				break;
			end			
			
			new_row = new_row+1;
			new_col = 1;
		end
		
		nUI:buttonBagInsert( bag, new_button, new_row, new_col );
	end
end

-------------------------------------------------------------------------------

function nUI:compressButtonBag()
	
	local new_bag = {};
	local row     = 1;
	local col     = 1;
	local old_bag = nUI.info_frame.buttonbag.buttons;
	
	if old_bag then
		
		for i=1,(nUI.info_frame.buttonbag.last_row or 0) do
			
			if old_bag[i] then
					
				for j=1,3 do
					
					if old_bag[i][j] then
						
						local button = old_bag[i][j];
						
						if not new_bag[row] then 
							new_bag[row] = {}; 
						end
			
						new_bag[row][col] = button;
						button.row = row;
						button.col = col;

						nUI_Config.ButtonBag.Buttons[button.name] = 
						{
							row = button.row,
							col = button.col,
						};
						
						if col == 3 then 
							col = 1;
							row = row+1;
						else
							col = col+1;
						end
					end
				end
			end
		end
	end
	
	nUI.info_frame.buttonbag.buttons = new_bag;
	
end
						
