﻿-------------------------------------------------------------------------------
-- local cache

local CreateFrame            = CreateFrame;
local GetFactionInfo         = GetFactionInfo;
local GetNumFactions         = GetNumFactions;
local GetWatchedFactionInfo  = GetWatchedFactionInfo;
local SetWatchedFactionIndex = SetWatchedFactionIndex;

nUI_FactionStandings =
{
	[0] = "Unknown",	
	[1] = "Hated",	
	[2] = "Hostile",	
	[3] = "Unfriendly",	
	[4] = "Neutral",	
	[5] = "Friendly",	
	[6] = "Honored",	
	[7] = "Revered",	
	[8] = "Exalted", 
};

-------------------------------------------------------------------------------

nUI.Class.FactionBar = nUI.Class:CreatePrototype( nUI.Class.StatusBar );
local prototype      = nUI.Class.FactionBar.prototype;

-------------------------------------------------------------------------------

function prototype:init( parent, id, layout )

	prototype.super.prototype.init( self, parent, id, layout );

	self.faction = GetWatchedFactionInfo();
	
	self.frame:SetPoint( self.layout.Origin.anchor, nUI_Dashboard, "CENTER", self.layout.Origin.xOfs, self.layout.Origin.yOfs );
	
	self.frame:SetBackdrop(
		{
			bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
			edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
			tile     = true, 
			tileSize = 1, 
			edgeSize = 12, 
			insets   = {left = 0, right = 0, top = 0, bottom = 0},
		}
	);

	self.frame:SetBackdropColor( 0, 0, 0, 0.35 );
	self.frame.grid = CreateFrame( "Frame", "$parent_Grid", self.frame );
	self.frame.grid:SetHeight( self.frame:GetHeight() - 2 );
	self.frame.grid:SetWidth( self.frame:GetWidth() - 2 );
	self.frame.grid:SetPoint( "CENTER", self.frame, "CENTER", 0, 0 );
	self.frame.grid:SetFrameLevel( self.frame.bar:GetFrameLevel()+1 );
	self.frame.grid.texture = self.frame.grid:CreateTexture();
	self.frame.grid.texture:SetAllPoints( self.frame.grid );
	self.frame.grid.texture:SetTexture( "Interface\\AddOns\\nUI\\Bars\\Art\\nUI_XPRepOverlay.blp" );
	self.frame.grid.texture:SetTexCoord( 0, 0, 0, 1, 0.978515625, 0, 0.978515625, 1 );
	
	local color = self.layout.Label.Color;
	
	self.frame.bar.center:SetScale( 1.0 / nUI.scale );
	self.frame.bar.center.text:ClearAllPoints();
	self.frame.bar.center.text:SetFont( "Fonts\\ARIALN.TTF", self.layout.Label.size * nUI.scale, "OUTLINE" );
	self.frame.bar.center.text:SetPoint( self.layout.Label.anchor, self.frame, "CENTER", self.layout.Label.xOfs * nUI.scale, self.layout.Label.yOfs * nUI.scale );
	self.frame.bar.center.text:SetTextColor( color.r, color.g, color.b, 1 );
	
	self:updateFaction();

	self.event_frame = CreateFrame( "Frame", self.layout.name.."_EventFrame", self.parent );
	self.event_frame:SetScript( "OnEvent", function() self:onEvent(); end );
	self.event_frame:RegisterEvent( "UPDATE_FACTION" );
	self.event_frame:RegisterEvent( "CHAT_MSG_COMBAT_FACTION_CHANGE" );
	
end

-------------------------------------------------------------------------------

function prototype:onEvent()
	
	if event == "UPDATE_FACTION" then
		self:updateFaction();
	elseif event == "CHAT_MSG_COMBAT_FACTION_CHANGE" then
		self:updateFaction( arg1 );
	end	
end

-------------------------------------------------------------------------------

function prototype:updateFaction( msg )

	local	track_faction;
	
	-- if we trapped a combat log chat message, then which faction was it talking about?
	
	if msg then

		local increase_idx = msg:find( " increased" );
		
		if not msg:match( "Reputation with " ) then
			
			return;
			
		elseif not increase_idx then
			
			return;
			
		else
			
			track_faction = msg:sub( 17, increase_idx-1 );
			
			-- if we have a new faction to track, make it the watched faction
			
			if track_faction and self.faction ~= track_faction then
				
				for i=1, GetNumFactions() do
					
					local name = GetFactionInfo( i );
					
					if name == track_faction then
		
						self.faction = track_faction;
						SetWatchedFactionIndex( i );
						
						break;
					end
				end
			end	
		end
	end
	
	-- if we don't have a faction, is there one we should be watching?
	
	if not self.faction then
		
		self.faction = GetWatchedFactionInfo();
		
	end
	
	-- if we still don't have a faction we're watching, then hide the bar
	
	if not self.faction then
		
		if self.frame:IsShown() then self:Hide(); end
		
	else

		if not self.frame:IsShown() then self.frame:Show(); end
		
		local name, standing, min, max, cur = GetWatchedFactionInfo();

		self:updateValues( cur, min, min, max );
		self.frame.bar.center.text:SetText( (name..": "..cur.."/"..max.." (%0.1f%% "..(nUI_FactionStandings[standing] or "standing")..")"):format( (cur-min)/(max-min)*100 ) );
		
	end
end