﻿
-------------------------------------------------------------------------------
-- local cache

local CreateFrame             = CreateFrame;
local ToggleDropDownMenu      = ToggleDropDownMenu;
local UnitIsUnit              = UnitIsUnit;
local UnitIsPlayer            = UnitIsPlayer;
local UnitInParty             = UnitInParty;
local UnitLevel               = UnitLevel;
local UnitReaction            = UnitReaction;
local UnitPowerType           = UnitPowerType;
local UnitExists              = UnitExists;
local UnitName                = UnitName;
local UnitClass               = UnitClass;
local UnitIsPVP               = UnitIsPVP;
local UnitClassification      = UnitClassification;
local UnitIsPVPFreeForAll     = UnitIsPVPFreeForAll;
local UnitIsVisible           = UnitIsVisible;
local UnitIsFriend            = UnitIsFriend;
local UnitPlayerControlled    = UnitPlayerControlled;
local UnitPlayerOrPetInParty  = UnitPlayerOrPetInParty;
local UnitPlayerOrPetInRaid   = UnitPlayerOrPetInRaid;
local UnitIsTapped            = UnitIsTapped;
local UnitIsTappedByPlayer    = UnitIsTappedByPlayer;
local UnitGUID                = UnitGUID;
local UnitIsPVPSanctuary      = UnitIsPVPSanctuary;
local SetPortraitTexture      = SetPortraitTexture;
local UnitIsConnected         = UnitIsConnected;
local UnitIsAFK               = UnitIsAFK;
local UnitIsDead              = UnitIsDead;
local UnitIsDeadOrGhost       = UnitIsDeadOrGhost;
local UIFrameFlash            = UIFrameFlash;
local GetPetHappiness         = GetPetHappiness;
local HasPetUI                = HasPetUI;
local UIFrameFlashRemoveFrame = UIFrameFlashRemoveFrame;
local IsResting               = IsResting;
local GetRaidTargetIndex      = GetRaidTargetIndex;
local UnitIsPartyLeader       = UnitIsPartyLeader;

nUI.Class.Unit  = nUI.Class:CreatePrototype();
local prototype = nUI.Class.Unit.prototype;

-------------------------------------------------------------------------------

function prototype:init( parent, id, layout )
	
	self.parent         = parent;
	self.id             = id;
	self.rid            = 1;
	self.initialized    = false;
	self.layout         = layout;
	self.is_shown       = true;
	self.Cache          = {};
	self.unit_name      = UnitName( self.layout.unit_id ) or "";	
	self.Timers         = {};
	self.Timers.refresh = 0;
	self.Timers.cam_fix = 0;
	
	if self.layout.Portrait == nil then
		self.layout.Portrait = {};
	end
	
	if self.layout.StatusBars == nil then
		self.layout.StatusBars = {};
	end
	
	if self.layout.BuffInfo == nil then
		self.layout.BuffInfo = {};
	end
	
	self:layoutFrame();
	
end

-------------------------------------------------------------------------------

function prototype:layoutFrame()

	local unit_id = self.layout.unit_id;
	
	if self.initialized ~= true then
		
		local dX = (self.layout.Size.width - 6) * nUI.scale;
		local dY = (self.layout.Size.height - 6) * nUI.scale;
		local info_box_size = math.floor( dY / 3 );
		local dL = dY - info_box_size * 2;
		local portrait;		
		
		-- calculate dimentions for the boxes based on the unit options
		
		if self.layout.Size.compact == "yes" then
			
			dL = dY;
			info_box_size = dL * 2 / 3;
			self.compact  = true;
			
		elseif self.layout.show_detail == "no" and self.layout.StatusBars.visible == "no" then
			
			dL            = dY;
			info_box_size = 0;
			
		elseif (self.layout.Size.two_line == "yes" or self.layout.show_class == "no") and self.layout.show_detail ~= "no" then
			
			info_box_size = math.floor( dY * 0.5 ) + 2;
			dL            = dY - info_box_size;
			
		end
		
		self.dY             = dY;
		self.dX             = dX;
		self.initialized    = true;
		self.frame_name     = (self.layout.name or "nUI_Unit").."_"..(unit_id or self.id);

		-- create the base frame and event handler
		
		self.event_frame = CreateFrame( "Button", self.frame_name.."_EventHandler", self.parent, "SecureUnitButtonTemplate" );
		self.event_frame.class    = self;
		self.event_frame.is_shown = true;
		self.event_frame.unit     = unit_id;

		self.event_frame:SetFrameStrata( "LOW" );
		self.event_frame:SetFrameLevel( 0 );
		self.event_frame:SetScale( 1.0 / nUI.scale );
		self.event_frame:SetWidth( self.layout.Size.width * nUI.scale );
		self.event_frame:SetHeight( self.layout.Size.height * nUI.scale );
		self.event_frame:SetPoint( self.layout.Origin.anchor, self.parent, "BOTTOM", self.layout.Origin.xOfs * nUI.scale, self.layout.Origin.yOfs * nUI.scale );
				
		-- the main backdrop frame for the entire unit

		self.frame          = CreateFrame( "Frame", self.frame_name, self.parent );
		
		self.frame.class    = self;
		self.frame.is_shown = true;
		
		self.frame:SetFrameStrata( "LOW" );
		self.frame:SetScale( 1.0 / nUI.scale );
		self.frame:SetFrameLevel( 1 );
		self.frame:SetAllPoints( self.event_frame );
	
		-- unit ID used in raids and groups
		
		if self.layout.id then self.frame:SetID( self.layout.id ); end
		
		-- create an optional border to outline where the unit is located
		
		if self.layout.show_border == "yes" then
			
			self.border = CreateFrame( "Frame", self.frame_name.."_Border", self.parent );

			self.border:SetFrameStrata( "LOW" );
			self.border:SetScale( 1.0 / nUI.scale );
			self.border:SetAllPoints( self.event_frame );
			self.border:SetFrameLevel( 0 );

			self.border:SetBackdrop(
				{
					bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
					edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 12, 
					insets   = {left = 0, right = 0, top = 0, bottom = 0},
				}
			);

			self.border:SetBackdropColor( 0, 0, 0, 0.35 );
			
		end
		
		-- if we're showing the portrait, then create the frame for that
		
		if not self.compact and self.layout.Portrait.visible ~= "no" then
			
			dX = dX - dY;
			
			self.frame.model = CreateFrame( "Button", self.frame_name.."_Model", self.frame );

			self.frame.model:SetFrameLevel( 3 );
			self.frame.model:SetBackdrop(
				{
					bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
					edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 10, 
					insets   = {left = 2, right = 2, top = 2, bottom = 2},
				}
			);
			self.frame.model:SetBackdropColor( 0, 0, 0, 0.75 );
			self.frame.model:SetWidth( dY );
			self.frame.model:SetHeight( dY );
			
			self:createUnitHitBox( self.frame.model );
			
			if self.layout.orientation == "left" then
				self.frame.model.hitbox:SetPoint( "TOPLEFT", self.event_frame, "TOPLEFT", 2, -2 );
			else
				self.frame.model.hitbox:SetPoint( "TOPRIGHT", self.event_frame, "TOPRIGHT", -2, -2 );
			end
			
			self.frame.model:SetAllPoints( self.frame.model.hitbox );
			
			-- the actual model animation or still portrait as the case may be
			
			self.Cache.portrait = CreateFrame( "Frame", self.frame_name.."_ModelPortrait", self.frame.model );
			
			self.Cache.portrait:SetFrameLevel( 4 );
			self.Cache.portrait:SetWidth( self.dY - 6 );
			self.Cache.portrait:SetHeight( self.dY - 6 );
			self.Cache.portrait:SetPoint( "TOPLEFT", self.frame.model, "TOPLEFT", 3, -3 );

			self.Cache.portrait.texture = self.Cache.portrait:CreateTexture();
			self.Cache.portrait.texture:SetAllPoints( self.Cache.portrait );
			self.Cache.portrait:Hide();

			self.Cache.anim = CreateFrame( "PlayerModel", self.frame_name.."_ModelAnim", self.frame.model );
			
			self.Cache.anim:SetFrameLevel( 4 );
			self.Cache.anim:SetWidth( self.dY - 6 );
			self.Cache.anim:SetHeight( self.dY - 6 );
			self.Cache.anim:SetPoint( "TOPLEFT", self.frame.model, "TOPLEFT", 3, -3 );
			self.Cache.anim:SetScript( "OnShow", function() self.frame.model.anim:SetCamera( 0 ); end );
			self.Cache.anim:SetUnit( unit_id );
			self.Cache.anim:Hide();
			
			-- a text box for describing the unit
			
			self.frame.model.info = CreateFrame( "Frame", self.frame_name.."_UnitType", self.frame.model );

			self.frame.model.info:SetFrameLevel( 5 );
			self.frame.model.info:SetBackdrop(
				{
					bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
					edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 10, 
					insets   = {left = 2, right = 2, top = 2, bottom = 2},
				}
			);
			self.frame.model.info:SetBackdropColor( 0, 0, 0, 0.75 );
			self.frame.model.info:SetWidth( dY+1 );
			self.frame.model.info:SetHeight( dY / 4 );
			self.frame.model.info:SetPoint( "BOTTOM", self.frame.model, "BOTTOM", 0, 0 );
			self.frame.model.info:Hide();

			self.frame.model.info.text = self.frame.model.info:CreateFontString( self.frame_name.."_UnitTypeText", "OVERLAY" );
			self.frame.model.info.text:SetPoint( "CENTER", self.frame.model.info, "CENTER", 0, 0 );
			self.frame.model.info.text:SetFont( "Fonts\\\ARIALN.TTF", (dY / 4 - 6) * 0.9, "OUTLINE" );
			self.frame.model.info.text:SetTextColor( 1, 1, 1, 1 );
			self.frame.model.info.text:SetJustifyH( "CENTER" );
			self.frame.model.info.text:SetJustifyV( "MIDDLE" );

			-- overlay frame for elites, rares and world bosses
	
			if self.layout.elite_icon == "yes" then
				self.frame.model.elite = CreateFrame( "Frame", self.frame_name.."_EliteIcon", self.frame.model );
				self.frame.model.elite:SetWidth( 256 );
				self.frame.model.elite:SetHeight( 256 );
				self.frame.model.elite:SetScale( (dY - 6) * 0.009 );
	
				self.frame.model.elite.texture = self.frame.model.elite:CreateTexture();
				self.frame.model.elite.texture:SetAllPoints( self.frame.model.elite );

				if self.layout.orientation == "left" then
					self.frame.model.elite:SetPoint( "CENTER", self.frame.model, "CENTER", -22, -3 );
				else
	    			self.frame.model.elite:SetPoint( "CENTER", self.frame.model, "CENTER", 22, -3 );
				end

				-- depending on whether the model is square or round, we either overlay the info
				-- over the elite icon (round models) or visa versa (square models)
				
				if self.layout.Portrait.style == "square" then
					self.frame.model.elite:SetFrameLevel( 6 );
					self.frame.model.info:SetFrameLevel( 5 );
				else
					self.frame.model.elite:SetFrameLevel( 5 );
					self.frame.model.info:SetFrameLevel( 6 );
				end            
			end

			-- combat feedback system

			self.frame.model.feedback = CreateFrame( "Frame", "$parent_Feedback", self.frame.model );
			self.frame.model.feedback.text = self.frame.model.feedback:CreateFontString( "$parentText" );
			self.frame.model.feedback.text:SetWidth( self.frame.model:GetWidth() );
			self.frame.model.feedback.text:SetHeight( self.frame.model:GetHeight() );
			self.frame.model.feedback.text:SetPoint( "CENTER", self.frame.model, "CENTER", -1, 0 );
			self.frame.model.feedback.text:SetFont( "Fonts\\skurri.ttf", self.dY * 0.29, "THICKOUTLINE" );
			self.frame.model.feedback.text:SetJustifyH( "CENTER" );
			self.frame.model.feedback.text:SetJustifyV( "MIDDLE" );
			self.frame.model.feedback:SetFrameLevel( 8 );

			-- combat and resting status
			
			self.frame.model.status = CreateFrame( "Frame", "$parent_Status", self.frame.model );
			self.frame.model.status:SetWidth( dL+2 );
			self.frame.model.status:SetHeight( dL+2 );
			self.frame.model.status:SetFrameLevel( 5 );
			self.frame.model.status:Hide();
			
			if self.layout.orientation == "left" then
				self.frame.model.status:SetPoint( "TOPLEFT", self.frame.model, "TOPLEFT", 0, 0 );
			else
				self.frame.model.status:SetPoint( "TOPRIGHT", self.frame.model, "TOPRIGHT", 0, 0 );
			end
			
			self.frame.model.status.texture = self.frame.model.status:CreateTexture();
			self.frame.model.status.texture:SetAllPoints( self.frame.model.status );
			self.frame.model.status.texture:SetTexture( "Interface\\AddOns\\nUI\\Units\\Art\\nUI_StateIcon.blp" );
			
		end

		-- frame for the name of the mob in the unit
		
		self.frame.label = CreateFrame( "Frame", "$parent_Label", self.frame );

		self.frame.label:SetFrameLevel( self.compact and 5 or 3 );
		self.frame.label:SetBackdrop(
			{
				bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
				tile     = true, 
				tileSize = 1, 
				edgeSize = 10, 
				insets   = {left = 2, right = 2, top = 2, bottom = 2},
			}
		);
		self.frame.label:SetBackdropColor( 0, 0, 0, self.compact and 0 or 0.75 );
        self.frame.label:SetWidth( dX+1 );
		self.frame.label:SetHeight( dL+2 );
		
		self.frame.label.text = self.frame.label:CreateFontString( "$parentText", "OVERLAY" );
		self.frame.label.text:SetFont( "Fonts\\\FRIZQT__.TTF", (dL+2) * (self.compact and 0.35 or 0.475), "OUTLINE" );
		self.frame.label.text:SetTextColor( 1, 0.8196079, 0, 1 );
		self.frame.label.text:SetJustifyH( "CENTER" );
		self.frame.label.text:SetJustifyV( "MIDDLE" );

		if self.compact then
			self.frame.label.text:SetPoint( "TOP", self.frame.label, "TOP", 0, -2 );
		else
			self.frame.label.text:SetPoint( "CENTER", self.frame.label, "CENTER", 0, self.compact and 1 or 0 );
		end
		
		self:createUnitHitBox( self.frame.label );

		if self.layout.orientation == "left" then
			self.frame.label.hitbox:SetPoint( "TOPRIGHT", self.event_frame, "TOPRIGHT", -2, -2 );
		else
			self.frame.label.hitbox:SetPoint( "TOPLEFT", self.event_frame, "TOPLEFT", 2, -2 );
		end		

		self.frame.label:SetAllPoints( self.frame.label.hitbox );

		-- unit PvP/FFA frame
		
		self.frame.pvp = CreateFrame( "Frame", "$parent_PVP", self.frame );
		self.frame.pvp:SetHeight( dL );
		self.frame.pvp:SetWidth( dL );
		self.frame.pvp:SetFrameLevel( 4 );
		self.frame.pvp:Hide();
		
		self.frame.pvp.texture = self.frame.pvp:CreateTexture();
		self.frame.pvp.texture:SetAllPoints( self.frame.pvp );
		
		if self.layout.orientation == "left" then
			
			self.frame.pvp:SetPoint( "CENTER", self.frame.label, "RIGHT", 0, 0 );
			
		else
			
			self.frame.pvp:SetPoint( "CENTER", self.frame.label, "LEFT", 0, 0 );
			
		end
		
		-- frame for indicating group leader/assist status
		
		self.frame.leader = CreateFrame( "Frame", "$parent_Leader", self.frame );
		self.frame.leader:SetFrameLevel( 5 );
		self.frame.leader:SetHeight( self.compact and (dL/3) or (dL/2) );
		self.frame.leader:SetWidth( self.compact and (dL/3) or (dL/2) );
		self.frame.leader:SetPoint( "TOP", self.frame.label, "TOP", 0, -2 );
		self.frame.leader:Hide();
		
		self.frame.leader.texture = self.frame.leader:CreateTexture();
		self.frame.leader.texture:SetAllPoints( self.frame.leader );
		
		-- frame for indicating master looter status
		
		self.frame.master_looter = CreateFrame( "Frame", "$parent_MasterLooter", self.frame );
		self.frame.master_looter:SetFrameLevel( 5 );
		self.frame.master_looter:SetHeight( self.compact and (dL/3) or (dL/2) );
		self.frame.master_looter:SetWidth( self.compact and (dL/3) or (dL/2) );
		self.frame.master_looter:Hide();

		if self.layout.orientation == "left" then
			self.frame.master_looter:SetPoint( "RIGHT", self.frame.leader, "LEFT", 0, 0 );
		else
			self.frame.master_looter:SetPoint( "LEFT", self.frame.leader, "RIGHT", 0, 0 );
		end
		
		self.frame.master_looter.texture = self.frame.master_looter:CreateTexture();
		self.frame.master_looter.texture:SetAllPoints( self.frame.master_looter );
		self.frame.master_looter.texture:SetTexture( "Interface\\GroupFrame\\UI-Group-MasterLooter.blp" );
		self.frame.master_looter.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
		
		-- frame for indicating main tank/off-tank status
		
		self.frame.tank = CreateFrame( "Frame", "$parent_TankIcon", self.frame );
		self.frame.tank:SetFrameLevel( 5 );
		self.frame.tank:SetHeight( self.compact and (dL/3) or (dL/2) );
		self.frame.tank:SetWidth( self.compact and (dL/3) or (dL/2) );
		self.frame.tank:Hide();

		if self.layout.orientation == "left" then
			self.frame.tank:SetPoint( "LEFT", self.frame.leader, "RIGHT", 0, 0 );
		else
			self.frame.tank:SetPoint( "RIGHT", self.frame.leader, "LEFT", 0, 0 );
		end
		
		self.frame.tank.texture = self.frame.master_looter:CreateTexture();
		self.frame.tank.texture:SetAllPoints( self.frame.master_looter );

		-- raid icon
		
		self.frame.raid_icon = CreateFrame( "Frame", "$parent_RaidIcon", self.frame );
		self.frame.raid_icon:SetFrameLevel( 7 );
		self.frame.raid_icon:SetHeight( dL );
		self.frame.raid_icon:SetWidth( dL );
		self.frame.raid_icon:Hide();

		self.frame.raid_icon.texture = self.frame.raid_icon:CreateTexture();
		self.frame.raid_icon.texture:SetHeight( dL );
		self.frame.raid_icon.texture:SetWidth( dL );
		self.frame.raid_icon.texture:SetAllPoints( self.frame.raid_icon );
		self.frame.raid_icon.texture:SetTexture( "Interface\\TargetingFrame\\UI-RaidTargetingIcons" );
		
		if self.layout.orientation == "left" then			
			self.frame.raid_icon:SetPoint( "CENTER", self.frame.label, "LEFT", 0, 0 );			
		else
			self.frame.raid_icon:SetPoint( "CENTER", self.frame.label, "RIGHT", 0, 0 );			
		end
		
		-- unit casting/channeling bar
		
		if not self.compact and self.layout.CastingBars.visible ~= "no" then

			local Cy = self.frame.label:GetHeight() - 6;
			local Hx = self.frame.label:GetWidth() - 6;
			
			if self.layout.CastingBars.latency == "yes" then Cy = Cy - 3; end

			self.layout.CastBar.height = Cy;
			self.layout.CastBar.width  = Hx;				
			self.frame.casting         = nUI.Class.UnitBar_Casting:new( self.frame.label, self.id+self.rid, self.layout.CastBar, unit_id );
			self.rid                   = self.rid + (self.frame.casting.rid or 1);					
			
			self.frame.casting:SetPoint( "TOPLEFT", self.frame.label, "TOPLEFT", 3, -3 );
			
			self.layout.ChannelBar.height = Cy;
			self.layout.ChannelBar.width  = Hx;				
			self.frame.channel            = nUI.Class.UnitBar_Channeling:new( self.frame.label, self.id+self.rid, self.layout.ChannelBar, unit_id );
			self.rid                      = self.rid + (self.frame.channel.rid or 1);					
			
			self.frame.channel:SetPoint( "TOPLEFT", self.frame.label, "TOPLEFT", 3, -3 );
		
			-- unit latency bar... cannot exist without a casting bar
			
			if self.layout.CastingBars.latency == "yes" then
				
				self.layout.LatencyBar.height = 3;
				self.layout.LatencyBar.width  = Hx;	
				self.frame.latency            = nUI.Class.UnitBar_Latency:new( self.frame.label, self.id+self.rid, self.layout.LatencyBar, unit_id );
				self.rid                      = self.rid + (self.frame.latency.rid or 1);					
				
				self.frame.latency:SetPoint( "TOPLEFT", self.layout.CastBar.name, "BOTTOMLEFT", 0, 0 );

			end
		end
		
		-- unit detail information
		
		if not self.compact and self.layout.show_detail ~= "no" then

			dX = dX - info_box_size + 2;
			
			-- frame for displaying the mob's level (or skull for bosses) and class
		
			self.frame.level = CreateFrame( "Frame", self.frame_name.."_Level", self.frame );

			self.frame.level:SetFrameLevel( 3 );
			self.frame.level:SetBackdrop(
				{
					bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
					edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 10, 
					insets   = {left = 2, right = 2, top = 2, bottom = 2},
				}
			);
			self.frame.level:SetBackdropColor( 0, 0, 0, 0.75 );
			self.frame.level:SetWidth( info_box_size );
			self.frame.level:SetHeight( info_box_size  );
			
			self.frame.level.text = self.frame.level:CreateFontString( self.frame_name.."_LevelText", "OVERLAY" );
			self.frame.level.text:SetPoint( "CENTER", self.frame.level, "CENTER", 0.75, 0.75 );
			self.frame.level.text:SetFont( "Fonts\\\ARIALN.TTF", info_box_size * 0.6, "OUTLINE" );
			self.frame.level.text:SetTextColor( 1, 1, 1, 1 );
			self.frame.level.text:SetJustifyH( "CENTER" );
			self.frame.level.text:SetJustifyV( "MIDDLE" );
	
			self:createUnitHitBox( self.frame.level );

			if self.layout.orientation == "left" then
				self.frame.level.hitbox:SetPoint( "TOPLEFT", self.frame.label.hitbox, "BOTTOMLEFT", 0, 1 );
			else
				self.frame.level.hitbox:SetPoint( "TOPRIGHT", self.frame.label.hitbox, "BOTTOMRIGHT", 0, 1 );
			end
			
			self.frame.level:SetAllPoints( self.frame.level.hitbox );
			
			-- a frame for displaying the boss icon for those mobs that require it
			
			self.frame.level.boss = CreateFrame( "Frame", self.frame_name.."_BossIcon", self.frame.level );

			self.frame.level.boss:SetFrameLevel( 4 );
			self.frame.level.boss:SetBackdrop(
				{
					bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
					edgeFile = nil, 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 10, 
					insets   = {left = 2, right = 2, top = 2, bottom = 2},
				}
			);
			self.frame.level.boss:SetBackdropColor( 0, 0, 0, 1 );
			self.frame.level.boss:SetPoint( "TOPLEFT", self.frame.level, "TOPLEFT", 3, -3 );
			self.frame.level.boss:SetPoint( "TOPRIGHT", self.frame.level, "TOPRIGHT", -3, -3 );
			self.frame.level.boss:SetPoint( "BOTTOMLEFT", self.frame.level, "BOTTOMRIGHT", 3, 3 );
			self.frame.level.boss:SetPoint( "BOTTOMRIGHT", self.frame.level, "BOTTOMRIGHT", -3, 3 );
	
			self.frame.level.boss.texture = self.frame.level.boss:CreateTexture();
			self.frame.level.boss.texture:SetTexture( "Interface\\TargetingFrame\\UI-TargetingFrame-Skull" );
			self.frame.level.boss.texture:SetAllPoints( self.frame.level.boss );
	
			-- frame for displaying the unit's class icon (warrior, shaman, etc.)
			
			if self.layout.show_class ~= "no" then
				
				self.frame.unit_class = CreateFrame( "Frame", self.frame_name.."_Class", self.frame );

				self.frame.unit_class:SetFrameLevel( 3 );
				self.frame.unit_class:SetBackdrop(
					{
						bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
						edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
						tile     = true, 
						tileSize = 1, 
						edgeSize = 10, 
						insets   = {left = 2, right = 2, top = 2, bottom = 2},
					}
				);
				self.frame.unit_class:SetBackdropColor( 0, 0, 0, 0.75 );
				self.frame.unit_class:SetWidth( info_box_size );
				self.frame.unit_class:SetHeight( info_box_size );
				
				self:createUnitHitBox( self.frame.unit_class );
	
				if self.layout.Size.two_line == "yes" then
					if self.layout.orientation ~= "right" then
						self.frame.unit_class.hitbox:SetPoint( "LEFT", self.frame.level.hitbox, "RIGHT", -1, 0 );
					else
						self.frame.unit_class.hitbox:SetPoint( "RIGHT", self.frame.level.hitbox, "LEFT", 1, 0 );
					end
					
					dX = dX - info_box_size + 1;
				else
					self.frame.unit_class.hitbox:SetPoint( "TOPLEFT", self.frame.level.hitbox, "BOTTOMLEFT", 0, 1 );
				end
							
				self.frame.unit_class:SetAllPoints( self.frame.unit_class.hitbox );

				self.frame.unit_class.texture = self.frame.unit_class:CreateTexture();
				self.frame.unit_class.texture:SetTexture( "Interface\\AddOns\\nUI\\Units\\Art\\nUI_Icon_Classes.blp" );
				self.frame.unit_class.texture:SetPoint( "TOPLEFT", self.frame.unit_class, "TOPLEFT", 3, -3 );
				self.frame.unit_class.texture:SetPoint( "TOPRIGHT", self.frame.unit_class, "TOPRIGHT", -3, -3 );
				self.frame.unit_class.texture:SetPoint( "BOTTOMLEFT", self.frame.unit_class, "BOTTOMLEFT", 3, 3 );
				self.frame.unit_class.texture:SetPoint( "BOTTOMRIGHT", self.frame.unit_class, "BOTTOMRIGHT", -3, 3 );
			end
		end
		
		-- create a pet happiness indicator
		
		self.frame.happiness = CreateFrame( "Frame", self.frame_name.."_Happiness", self.frame );

		self.frame.happiness:SetFrameLevel( 3 );
		self.frame.happiness:SetBackdrop(
			{
				bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
				tile     = true, 
				tileSize = 1, 
				edgeSize = 10, 
				insets   = {left = 2, right = 2, top = 2, bottom = 2},
			}
		);
		self.frame.happiness:SetBackdropColor( 0, 0, 0, 0.75 );
		self.frame.happiness:SetWidth( dL + 2 );
		self.frame.happiness:SetHeight( dL + 2 );
		self.frame.happiness:SetFrameLevel( 5 );
		self.frame.happiness:Hide();

		self.frame.happiness.texture = self.frame.happiness:CreateTexture();
		self.frame.happiness.texture:SetPoint( "TOPLEFT", self.frame.happiness, "TOPLEFT", 3, -3 );
		self.frame.happiness.texture:SetPoint( "TOPRIGHT", self.frame.happiness, "TOPRIGHT", -3, -3 );
		self.frame.happiness.texture:SetPoint( "BOTTOMLEFT", self.frame.happiness, "BOTTOMLEFT", 3, 3 );
		self.frame.happiness.texture:SetPoint( "BOTTOMRIGHT", self.frame.happiness, "BOTTOMRIGHT", -3, 3 );
		self.frame.happiness.texture:SetTexture( "Interface\\PetPaperDollFrame\\UI-PetHappiness.blp" );
		
		if self.frame.model then				
			if self.layout.orientation ~= "right" then
				self.frame.happiness:SetPoint( "TOPRIGHT", self.frame.model, "TOPRIGHT", 0, 0 );
			else
				self.frame.happiness:SetPoint( "TOPLEFT", self.frame.model, "TOPLEFT", 0, 0 );
			end
		else
			if self.layout.orientation ~= "right" then
				self.frame.happiness:SetPoint( "LEFT", self.frame.label, "RIGHT", 0, 0 );
			else
				self.frame.happiness:SetPoint( "RIGHT", self.frame.label, "LEFT", 0, 0 );
			end
		end
		
		-- create the status bar box if it is to be displayed
		
		if self.compact or self.layout.StatusBars.visible == "yes" then
	
			self.frame.status = CreateFrame( "Frame", self.frame_name.."_Status", self.frame );

			self.frame.status:SetFrameLevel( 3 );
			self.frame.status:SetBackdrop(
				{
					bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", 
					edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
					tile     = true, 
					tileSize = 1, 
					edgeSize = 10, 
					insets   = {left = 2, right = 2, top = 2, bottom = 2},
				}
			);
			self.frame.status:SetBackdropColor( 0, 0, 0, 0.75 );
			self.frame.status:SetWidth( dX+1 );
			
			if self.compact then
				self.frame.status:SetHeight( info_box_size );
			elseif self.layout.Size.two_line == "yes" and self.layout.show_detail ~= "no" then
				self.frame.status:SetHeight( info_box_size );
			else
				self.frame.status:SetHeight( info_box_size * 2 );
			end

			self:createUnitHitBox( self.frame.status );

			if self.compact then
				self.frame.status.hitbox:SetPoint( "BOTTOM", self.frame.label.hitbox, "BOTTOM", 0, 1 );
			elseif self.layout.orientation == "left" then
				self.frame.status.hitbox:SetPoint( "TOPRIGHT", self.frame.label.hitbox, "BOTTOMRIGHT", 0, 1 );
			else
				self.frame.status.hitbox:SetPoint( "TOPLEFT", self.frame.label.hitbox, "BOTTOMLEFT", 0, 1 );
			end
			
			self.frame.status:SetAllPoints( self.frame.status.hitbox );
			
			-- calculate how we're fitting bars inside the box
			
			local	Hx   = dX - 4;
			local	Hy   = info_box_size - 4;
			local	Py   = 0;		-- height of the power bar
			local	bars = 1;		-- the nunmber of bars to display health/power/casting
			
			if self.layout.StatusBars.power ~= "none" then
				bars = bars+1;
			end
			
			if not self.compact and (self.layout.Size.two_line ~= "yes" or self.layout.show_detail == "no") then
				Hy = Hy + info_box_size;
			end
			
			if self.layout.StatusBars.power == "thin" then
				Py = math.floor( Hy / bars * 0.5 );
			elseif self.layout.StatusBars.power ~= "none" then
				Py = math.floor( Hy / bars );
			end
			
			Hy = Hy - Py;

			-- create the status bars as required
			
			self.frame.status.power   = nil;
			self.frame.status.latency = nil;

			self.layout.HealthBar.width  = Hx;
			self.layout.HealthBar.height = Hy;
			self.frame.status.health     = nUI.Class.UnitBar_Health:new( self.frame.status, self.id+self.rid, self.layout.HealthBar, unit_id );
			self.rid                     = self.rid + (self.frame.status.health.rid or 1);

			local	hook = self.layout.HealthBar.name;
			
			self.frame.status.health:SetPoint( "TOPLEFT", self.frame.status, "TOPLEFT", 2, -3 );

			if self.layout.StatusBars.power ~= "none" then
				
				self.layout.ManaBar.width    = Hx;
				self.layout.ManaBar.height   = Py;
				self.frame.status.mana       = nUI.Class.UnitBar_Mana:new( self.frame.status, self.id+self.rid, self.layout.ManaBar, unit_id );
				self.rid                     = self.rid + (self.frame.status.mana.rid or 1);
				
				self.layout.RageBar.width    = Hx;
				self.layout.RageBar.height   = Py;
				self.frame.status.rage       = nUI.Class.UnitBar_Rage:new( self.frame.status, self.id+self.rid, self.layout.RageBar, unit_id );
				self.rid                     = self.rid + (self.frame.status.rage.rid or 1);
				
				self.layout.EnergyBar.width  = Hx;
				self.layout.EnergyBar.height = Py;
				self.frame.status.energy     = nUI.Class.UnitBar_Energy:new( self.frame.status, self.id+self.rid, self.layout.EnergyBar, unit_id );
				self.rid                     = self.rid + (self.frame.status.energy.rid or 1);

				self.layout.FocusBar.width   = Hx;
				self.layout.FocusBar.height  = Py;
				self.frame.status.focus      = nUI.Class.UnitBar_Focus:new( self.frame.status, self.id+self.rid, self.layout.FocusBar, unit_id );
				self.rid                     = self.rid + (self.frame.status.focus.rid or 1);
				
				self.frame.status.mana:SetPoint( "TOPLEFT", hook, "BOTTOMLEFT", 0, 0 );
				self.frame.status.rage:SetPoint( "TOPLEFT", hook, "BOTTOMLEFT", 0, 0 );
				self.frame.status.focus:SetPoint( "TOPLEFT", hook, "BOTTOMLEFT", 0, 0 );
				self.frame.status.energy:SetPoint( "TOPLEFT", hook, "BOTTOMLEFT", 0, 0 );
				
				self.frame.status.mana:Hide();
				self.frame.status.rage:Hide();
				self.frame.status.focus:Hide();
				self.frame.status.energy:Hide();

				hook = self.layout.ManaBar.name;
				
			end
		end

		-- create a state overlay frame used to indicate AFK / Dead / etc.
		
		self.frame.state = CreateFrame( "Frame", self.frame_name.."_State", self.frame );
		self.frame.state:SetBackdrop(
			{
				bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
				edgeFile = nil, 
				tile = true, 
				tileSize = 1, 
				edgeSize = 10, 
				insets = {left = 2, right = 2, top = 2, bottom = 2},
			}
		);
		self.frame.state:SetFrameLevel( 8 );
		self.frame.state:SetBackdropColor( 0.5, 0.5, 0.5, 0.75 );
		self.frame.state:SetWidth( self.layout.Size.width * nUI.scale - 4 );
		self.frame.state:SetHeight( self.layout.Size.height * nUI.scale - 4 );
		self.frame.state:SetPoint( "TOPLEFT", self.frame, "TOPLEFT", 2, -2 );

		self.frame.state.text = self.frame.state:CreateFontString( self.frame_name.."_StateText", "OVERLAY" );
		self.frame.state.text:SetFont( "Fonts\\\ARIALN.TTF", dL * 0.95, "OUTLINE" );
		self.frame.state.text:SetTextColor( 1, 0.25, 0.25, 1 );
		self.frame.state.text:SetJustifyH( "CENTER" );
		self.frame.state.text:SetJustifyV( "MIDDLE" );
		
		if self.layout.orientation == "left" then
			self.frame.state.text:SetPoint( "RIGHT", self.frame.label, "RIGHT", -5, -dL * 0.025 );
		else
			self.frame.state.text:SetPoint( "LEFT", self.frame.label, "LEFT", 5,  -dL * 0.025 );
		end

		self.frame.state:Hide();
				
		-- layout the unit buffs
		
		self.frame.buffs = nUI.Class.Buffs:new( 
			self.frame, self.id, 
			self:createBuffLayout( self.layout.BuffInfo.Buffs, "Buffs" ), 
			unit_id 
		);

		self:setBuffLocation( self.frame.buffs, self.layout.BuffInfo.Buffs );
		
		self.frame.debuffs = nUI.Class.Buffs:new( 
			self.frame, self.id, 
			self:createBuffLayout( self.layout.BuffInfo.Debuffs, "Debuffs" ), 
			unit_id 
		);

		self:setBuffLocation( self.frame.debuffs, self.layout.BuffInfo.Debuffs );
		
		-- create a layer for the unit popup
		
		self.frame.popup = CreateFrame( "Frame", self.frame_name.."_Popup", self.frame, "UIDropDownMenuTemplate" );
		self.frame.popup.class = self;
	
		self.frame.popup:SetFrameLevel( 9 );
		self.frame.popup:SetPoint( "CENTER", self.frame, "CENTER", 0, 0 );
		
		self.frame.popup.texture = self.frame.popup:CreateTexture();
		self.frame.popup.texture:SetAllPoints( self.frame.popup );
		self.frame.popup.texture:SetTexture( 0, 0, 0, 0.75 );

		self.frame.popup:Hide();

		UIDropDownMenu_Initialize( self.frame.popup, function () 
			self:updateUnitPopup();
		end, "MENU" );
		
		self:registerEvents();
		
	end

	self:setHighlightNormal();
	self:Hide();

end

-------------------------------------------------------------------------------

function prototype:createBuffLayout( layout, type )
	
	local buff_layout;
	local visible = "yes";
	local model_side = self.layout.orientation;
	local cols       = math.ceil( 40 / layout.rows );
	local orient;
	local grow;
	
	if layout.location == "hidden" then visible = "hidden";
	elseif layout.location == "none" then visible = "no";
	end
	
	if layout.location == "detatched" then
		orient = layout.anchor;
		grow   = "horizontal";
	elseif layout.location == "left" then
		orient = "TOPRIGHT";
		grow   = "vertical";
	elseif layout.location == "right" then
		orient = "TOPLEFT";
		grow   = "vertical";
	elseif layout.location == "top" then
		orient = (model_side == "left" and "BOTTOMLEFT" or "BOTTOMRIGHT");
		grow   = "horizontal";
	else
		orient = (model_side == "left" and "TOPLEFT" or "TOPRIGHT");
		grow   = "horizontal";
	end
		
	buff_layout =
	{
		name       = self.frame_name.."_"..type,
		visible    = visible,
		flash      = layout.flash,
		flash_rate = layout.flash_rate,
		timers     = layout.timers,
		anim       = layout.anim,
		highlight  = layout.highlight,
		type       = layout.type,
		
		Shape =
		{
			orient = orient,
			grow   = grow,
			rows   = layout.rows,
			size   = (layout.width or self.layout.Size.width) / cols * nUI.scale,
			gap    = layout.gap,
		},
	};
	
	return nUI.Class.BuffsLayout:new( buff_layout );
		
end

-------------------------------------------------------------------------------

function prototype:setBuffLocation( frame, layout )
	
	local location = layout.location;
	local side     = self.layout.orientation;
	
	if location == "detatched" then
		
		frame:SetPoint( layout.anchor, nUI_Dashboard, "CENTER", layout.xOfs * nUI.scale, layout.yOfs * nUI.scale );
		
	elseif location == "left" then
		
		frame:SetPoint( "TOPRIGHT", self.frame, "TOPLEFT", 0, -3 );
		
	elseif location == "right" then
		
		frame:SetPoint( "TOPLEFT", self.frame, "TOPRIGHT", 0, -3 );
		
	elseif location == "top" then
		
		frame:SetPoint( "BOTTOMLEFT", self.frame, "TOPLEFT", (side == "left" and 3 or -3), 0 );
		
	else
		
		frame:SetPoint( "TOPLEFT", self.frame, "BOTTOMLEFT", (side == "left" and 3 or -3), 0 );
		
	end
	
end

-------------------------------------------------------------------------------

function prototype:createUnitHitBox( frame )

	local unit_id = self.layout.unit_id;
	
	frame.hitbox = CreateFrame( "Button", frame:GetName().."_HitBox", self.event_frame, "SecureUnitButtonTemplate" );
	frame.hitbox.unit = unit_id;
	
	frame.hitbox:SetHeight( frame:GetHeight() );
	frame.hitbox:SetWidth( frame:GetWidth() );
	frame.hitbox:SetFrameLevel( 8 );
	
	frame.hitbox:RegisterForClicks( "LeftButtonUp", "RightButtonUp" );	
	frame.hitbox:SetScript( "OnEnter", function() UnitFrame_OnEnter( frame.hitbox ); end );
	frame.hitbox:SetScript( "OnLeave", function() UnitFrame_OnLeave( frame.hitbox ); end );

	local showUnitPopup = function()
		ToggleDropDownMenu( 1, nil, self.frame.popup, self.event_frame, 106, 27 );
	end

	SecureUnitButton_OnLoad( frame.hitbox, unit_id, showUnitPopup );

end

-------------------------------------------------------------------------------

function prototype:SetPoint( point, relative_to, relative_point, ofsX, ofsY )
	
	self.event_frame:SetPoint( point, relative_to, relative_point, ofsX, ofsY );
	
end

function prototype:SetAllPoints( relative_to )
	
	self.event_frame:SetAllPoints( relative_to );
	
end

function prototype:ClearAllPoints()
	
	self.event_frame:ClearAllPoints();

end

-------------------------------------------------------------------------------

function prototype:Show()

--	if not self.is_shown then
		
		self.frame:Show();
	
		if self.frame.status then
			if self.frame.status.health then self.frame.status.health:Show(); end
			if self.frame.status.power then self.frame.status.power:Show(); end
			if self.frame.status.latency then self.frame.status.latency:Show(); end
		end
	
		self.is_shown = true;
--	end	
end	

-------------------------------------------------------------------------------

function prototype:Hide()

--	if self.is_shown then
		
		self.frame:Hide();
		
		if self.frame.status then
			if self.frame.status.health then self.frame.status.health:Hide(); end
			if self.frame.status.power then self.frame.status.power:Hide(); end
			if self.frame.status.latency then self.frame.status.latency:Hide(); end
		end
	
		self.is_shown = false;
--	end
end

-------------------------------------------------------------------------------

function prototype:SetBackdropColor( r, g, b, a )

	if self.frame.model then self.frame.model:SetBackdropColor( r, g, b, a ); end;
	if self.frame.level then self.frame.level:SetBackdropColor( r, g, b, a ); end;
	if self.frame.level and self.frame.level.boss then self.frame.level.boss:SetBackdropColor( r, g, b, a ); end;
	if self.frame.unit_class then self.frame.unit_class:SetBackdropColor( r, g, b, a ); end;
	if self.frame.status then self.frame.status:SetBackdropColor( r, g, b, a ); end;
	
end

function prototype:SetBackdropBorderColor( r, g, b, a )
	
	if self.frame.model then self.frame.model:SetBackdropBorderColor( r, g, b, a ); end;
	if self.frame.level then self.frame.level:SetBackdropBorderColor( r, g, b, a ); end;
	if self.frame.unit_class then self.frame.unit_class:SetBackdropBorderColor( r, g, b, a ); end;
	if self.frame.status then self.frame.status:SetBackdropBorderColor( r, g, b, a ); end;
	
	self.frame.label:SetBackdropBorderColor( r, g, b, a ); 

end

-------------------------------------------------------------------------------

function prototype:setHighlightAgro()
	
	self:SetBackdropColor( 0.5, 0, 0, 1 );
	self:SetBackdropBorderColor( 1, 0.3, 0.3, 0.75 );
	
end

function prototype:setHighlightHeal()
	
	self:SetBackdropColor( 0, 0.75, 0, 1 );
	self:SetBackdropBorderColor( 0.2, 1, 0.2, 0.75 );
	
end

function prototype:setHighlightCurse()
	
	self:SetBackdropColor( 0, 0.5, 0.5, 1 );
	self:SetBackdropBorderColor( .25, 1, 1, 0.75 );
	
end

function prototype:setHighlightNormal()
	
	self:SetBackdropColor( 0, 0, 0, 0.75 );
	self:SetBackdropBorderColor( 1, 1, 1, 0.75 );
	
end

-------------------------------------------------------------------------------

function prototype:destroyUnit()

	self.frame:SetScript( "OnUpdate", nil );
	
	self:Hide();
	
	self.frame       = nil;
	self.unit_name   = nil;
	self.guid        = nil;
	self.frame_name  = nil;
	self.initialized = false;
	
	collectgarbage();

end

-------------------------------------------------------------------------------

function prototype:enableAnim( mode )

	if mode == "no" then self.layout.Portrait.visible = "yes";
	else self.layout.Portrait.visible = "anim";
	end

	self:updateModel();
	
end

-------------------------------------------------------------------------------

function prototype:getRaidInfo()
	
	self.RaidInfo = {};

	self.RaidInfo.in_raid = UnitInRaid( self.layout.unit_id );
	
	if self.RaidInfo.in_raid then
			
		self.RaidInfo.name,self.RaidInfo.rank,self.RaidInfo.subGroup,self.RaidInfo.level,
		self.RaidInfo.class,self.RaidInfo.fileName,self.RaidInfo.zone,self.RaidInfo.online,
		self.RaidInfo.isDead,self.RaidInfo.role,self.RaidInfo.is_ml = GetRaidRosterInfo( self.frame:GetID() );
		
	end
end

-------------------------------------------------------------------------------
--                               EVENT HANDLING                              --
-------------------------------------------------------------------------------

function prototype:registerEvents()

	-- callbacks
	
	self.event_frame:SetScript( "OnEvent", function() self:onEvent(); end );
	self.event_frame:SetScript( "OnUpdate", function( who, elapsed ) self:onUpdate( elapsed ); end );

	-- events
	
	self.event_frame:RegisterEvent( "UNIT_DISPLAYPOWER" );
	self.event_frame:RegisterEvent( "UNIT_LEVEL" );
	self.event_frame:RegisterEvent( "UNIT_MODEL_CHANGED" );
	self.event_frame:RegisterEvent( "UNIT_NAME_UPDATE" );
	self.event_frame:RegisterEvent( "UNIT_PORTRAIT_UPDATE" );
	self.event_frame:RegisterEvent( "UNIT_COMBAT" );	
	self.event_frame:RegisterEvent( "UNIT_HAPPINESS" );
	self.event_frame:RegisterEvent( "UNIT_MAXHAPPINESS" );
	self.event_frame:RegisterEvent( "UNIT_FACTION" );
	self.event_frame:RegisterEvent( "UNIT_DYNAMIC_FLAGS" );
	self.event_frame:RegisterEvent( "UNIT_FLAGS" );
	self.event_frame:RegisterEvent( "UNIT_TARGET" );
	self.event_frame:RegisterEvent( "PARTY_MEMBERS_CHANGED" );	
	self.event_frame:RegisterEvent( "RAID_ROSTER_UPDATE" );	
	self.event_frame:RegisterEvent( "RAID_TARGET_UPDATE" );	
	self.event_frame:RegisterEvent( "UNIT_PET" );	
	self.event_frame:RegisterEvent( "PET_UI_UPDATE" );	
	self.event_frame:RegisterEvent( "PLAYER_FLAGS_CHANGED" );	
	self.event_frame:RegisterEvent( "PLAYER_ALIVE" );	
	self.event_frame:RegisterEvent( "PLAYER_DEAD" );	
	self.event_frame:RegisterEvent( "PLAYER_UNGHOST" );	
	self.event_frame:RegisterEvent( "PLAYER_TARGET_CHANGED" );	
	self.event_frame:RegisterEvent( "PLAYER_FOCUS_CHANGED" );	
	self.event_frame:RegisterEvent( "PLAYER_UPDATE_RESTING" );	
	self.event_frame:RegisterEvent( "PLAYER_ENTER_COMBAT" );	
	self.event_frame:RegisterEvent( "PLAYER_LEAVE_COMBAT" );	
	self.event_frame:RegisterEvent( "PLAYER_REGEN_DISABLED" );	
	self.event_frame:RegisterEvent( "PLAYER_REGEN_ENABLED" );	
	self.event_frame:RegisterEvent( "PET_ATTACK_START" );	
	self.event_frame:RegisterEvent( "PET_ATTACK_STOP" );	

	-- set up combat feedback on those units that have models

	if self.frame.model then
		
		self.CombatFeedback = function( arg2, arg3, arg4, arg5 )
			this = self.frame.model;
			CombatFeedback_OnCombatEvent( arg2, arg3, arg4, arg5 );
		end
				
		local initFeedback = function()
			this = self.frame.model;
			CombatFeedback_Initialize( self.frame.model.feedback.text, self.dY * 0.29 );
		end;
		
		initFeedback();
		
		self.frame.model.feedbackStartTime = 0;		
		self.frame.model:SetScript( "OnUpdate", CombatFeedback_OnUpdate );
				
	end		
end

-------------------------------------------------------------------------------

function prototype:onEvent()
	
	local unit_id = self.layout.unit_id;
	
	-- unit specific events...

	if     event == "PARTY_MEMBERS_CHANGED" then self:checkGUID();
	elseif event == "RAID_ROSTER_UPDATE"    then self:checkGUID();
	elseif event == "PET_UI_UPDATE"         then self:checkGUID();
	elseif event == "PLAYER_TARGET_CHANGED" then self:checkGUID();
	elseif event == "UNIT_TARGET"           then self:checkGUID();
	elseif event == "PLAYER_FOCUS_CHANGED"  then self:checkGUID();
	elseif event == "RAID_TARGET_UPDATE"    then self:updateRaidTarget();
	elseif event == "PLAYER_UPDATE_RESTING" then self:updateResting();
	
	elseif UnitIsUnit( unit_id, (arg1 or unit_id) ) then
		if     event == "UNIT_DISPLAYPOWER"     then self:updatePowerBar(); 
		elseif event == "UNIT_LEVEL"            then self:updateUnitLevel(); 
		elseif event == "UNIT_HAPPINESS"        then self:updateHappiness(); 
		elseif event == "UNIT_MAXHAPPINESS"     then self:updateHappiness(); 
		elseif event == "UNIT_PORTRAIT_UPDATE"  then self:updateModel();
		elseif event == "UNIT_PET"              then self:checkGUID();
		elseif event == "UNIT_NAME_UPDATE"      then self:checkGUID();
		elseif event == "UNIT_FACTION"          then self:updateFaction();
		elseif event == "UNIT_COMBAT"           then self:updateCombatStatus( arg2, arg3, arg4, arg5 );
		elseif event == "UNIT_MODELCHANGED"     then self:updateModel();
		elseif event == "UNIT_DYNAMIC_FLAGS"    then self:updateStatus();
		elseif event == "UNIT_FLAGS"            then self:updateStatus();
		elseif event == "PLAYER_FLAGS_CHANGED"  then self:updateStatus();
		
		elseif UnitIsUnit( "player", unit_id ) then
			if     event == "PLAYER_UPDATE_RESTING" then self:updateResting();
			elseif event == "PLAYER_ENTER_COMBAT"	then self:enterCombat();
			elseif event == "PLAYER_REGEN_DISABLED" then self:enterCombat();
			elseif event == "PLAYER_LEAVE_COMBAT"   then self:leaveCombat();
			elseif event == "PLAYER_REGEN_ENABLED"  then self:leaveCombat();
			elseif event == "PLAYER_ALIVE"          then self:updateStatus();
			elseif event == "PLAYER_DEAD"           then self:updateStatus();
			elseif event == "PLAYER_UNGHOST"        then self:updateStatus();
			end		
		
		elseif UnitIsUnit( "pet", unit_id ) then
			
			if event     == "PET_ATTACK_START" then self:enterCombat();
			elseif event == "PET_ATTACK_STOP" then self:leaveCombat();
			end
		end
	end
end

-------------------------------------------------------------------------------

function prototype:onUpdate( elapsed )
	
	self.Timers.refresh = self.Timers.refresh + elapsed;

	if self.parent:IsVisible() then
		
		-- update unit frame visibility at 12.5fps

		if self.Timers.refresh > 0.08 then
	
			self.Timers.refresh = 0;		
			self:refreshUnit();
		end
	end		
end

-------------------------------------------------------------------------------

function prototype:updateUnitPopup()

	local unit_id    = self.layout.unit_id;
	local unit_popup = nil;
	local raid_id    = nil;
	local unit_name  = nil;
	
	-- select the popup menu to be displayed
	
	if UnitIsUnit( "player", unit_id ) then
		
		unit_popup = "SELF";
		
	elseif UnitIsUnit( "pet", unit_id ) then
		
		unit_popup = "PET";

	elseif UnitIsPlayer( unit_id ) then
		
		raid_id = UnitInRaid( unit_id );
		
		if raid_id then
			
			unit_popup = "RAID_PLAYER";
			unit_name  = self.unit_name;
			
		elseif UnitInParty( unit_id ) then
			
			unit_popup = "PARTY";
			
		else
			
			unit_popup = "PLAYER";
			
		end
	else
		unit_popup = "RAID_TARGET_ICON";
		unit_name  = RAID_TARGET_ICON;
	end
	
	-- if we have one, then enable it
	
	if unit_popup then
		
		UnitPopup_ShowMenu( self.frame.popup, unit_popup, unit_id, unit_name, raid_id );
	
	end
end

-------------------------------------------------------------------------------

function prototype:updateUnitLevel()

	local	is_a_ding = false;
	local	new_level = UnitLevel( self.layout.unit_id );

	if self.unit_level ~= nil and self.unit_level ~= new_level and self.layout.unit_id == "player" then
		is_a_ding = true;
	end
	
	self.unit_level = new_level;
	
	if self.frame.level then
		
		self.frame.level.text:SetText( self.unit_level ); 

		-- if the unit is too high level, display the boss icon instead of the level
		
		if self.frame.level then
			if self.unit_level == -1 then	self.frame.level.boss:Show();
			else							self.frame.level.boss:Hide();
			end
		end
		
		-- color the level text according to the differnce between the player and the unit
		-- based also on the unit's reaction to the player
				
		local level_dif = UnitLevel( "player" ) - self.unit_level;

		if self.unit_level == -1 then level_dif = -5; end
								
		if self.is_friend then			self.frame.level.text:SetTextColor( 1, 0.8196079, 0, 1 );
		elseif level_dif >= 5 then		self.frame.level.text:SetTextColor( 0.65, 0.65, 0.65, 1 );
		elseif level_dif >= 3 then		self.frame.level.text:SetTextColor( 0, 1, 0, 1 );
		elseif level_dif >= -2 then		self.frame.level.text:SetTextColor( 1, 0.8196079, 0, 1 );
		else							self.frame.level.text:SetTextColor( 1, 0.25, 0.25, 1 );
		end
		
		-- auto ding option
		
		if is_a_ding and nUI_Config.auto_ding == "yes" then
			nUI:autoDing( new_level );
		end
	end		
end
	
-------------------------------------------------------------------------------

function prototype:updatePowerBar()

	-- update health and power bars as required
	
	if self.frame.status then

		self.frame.status.health:Show();

		local power_type = UnitPowerType( self.layout.unit_id );
			
		if self.frame.status.power ~= nil then
			self.frame.status.power:Hide();
			self.frame.status.power = nil;
		end

		if power_type == 0 then
			self.frame.status.power = self.frame.status.mana;
		elseif power_type == 1 then
			self.frame.status.power = self.frame.status.rage;
		elseif power_type == 2 then
			self.frame.status.power = self.frame.status.focus;
		elseif power_type == 3 then
			self.frame.status.power = self.frame.status.energy;
		else
			nUI:debugMsg( 1, "unhandled unit power type ["..power_type.."] for unit ["..self.layout.unit_id.."]" );
		end

		if self.frame.status.power ~= nil then
			self.frame.status.power:Show();
		end
	end
end

-------------------------------------------------------------------------------

function prototype:updateHappiness()

	local unit_id = self.layout.unit_id;
	
	-- if this unit doesn't exist or is not the player's pet, hide the happiness indicator
	
	if not UnitExists( unit_id ) or not UnitIsUnit( "pet", unit_id ) then

		self.happiness = nil;
		self.loyalty   = nil;
		
		if self.frame.happiness then
			
			UIFrameFlashRemoveFrame( self.frame.happiness );
			self.frame.happiness:Hide();
				
		end
	
	-- otherwise... update status as appropriate
	
	else

		local happiness, _, loyalty = GetPetHappiness();
		local _, hunter_pet         = HasPetUI();

		-- this only applies to hunter pets
		
		if not happiness or not hunter_pet then
				
			self.happiness = nil;
			self.loyalty   = nil;

			if self.frame.happiness and self.frame.happiness:IsShown() then
				
				UIFrameFlashRemoveFrame( self.frame.happiness );
				self.frame.happiness:Hide();
			end

		-- this is the player's hunter pet...
		
		else
			
			-- make sure we can see the indicator
			
			if self.frame.happiness and not self.frame.happiness:IsShown() then
				
				self.frame.happiness:SetAlpha( 1 );
				self.frame.happiness:Show();
				
			end
			
			-- update the player if the pet's happiness status changes
			
			if happiness ~= self.happiness then

				-- if this is the player's pet unit frame, then announce the change in chat
				
				if unit_id == "pet" then

					if happiness == 1 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: Warning! "..self.unit_name.." is |cFFFFFFFFNOT|r happy! Better feed soon.", 1, 0.5, 0.5 );
						
					elseif happiness == 2 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: "..self.unit_name.." is unhappy... time to feed!", 1, 0.83, 0 );
						
					elseif happiness == 3 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: "..self.unit_name.." is happy.", 0.5, 1, 0.5 );
						
					end
				end
				
				if self.frame.happiness then
					
					UIFrameFlashRemoveFrame( self.frame.happiness );
				
					self.frame.happiness:SetAlpha( 1 );
					self.frame.happiness:Show();

					if happiness == 1 then
						if unit_id == "pet" then
							self.frame.happiness:SetScale( 1.75 );
							UIFrameFlash( self.frame.happiness, 0.25, 0.25, 86400, true, 0.25, 0.25 );
						end
						self.frame.happiness.texture:SetTexCoord(0.375, 0.5625, 0, 0.359375);
					elseif happiness == 2 then
						if unit_id == "pet" then
							self.frame.happiness:SetScale( 1.5 );
							UIFrameFlash( self.frame.happiness, 0.25, 0.25, 30, true, 0.25, 0.25 );
						end
						self.frame.happiness.texture:SetTexCoord(0.1875, 0.375, 0, 0.359375);
					elseif happiness == 3 then
						if unit_id == "pet" then
							self.frame.happiness:SetScale( 1 );
							UIFrameFlash( self.frame.happiness, 0.25, 0.25, 10, true, 0.25, 0.25 );
						end
						self.frame.happiness.texture:SetTexCoord(0, 0.1875, 0, 0.359375);
					end
				end
				
				self.happiness = happiness;
				
			end

			-- update the player if the pet's loyalty status changes
			
			if loyalty ~= self.loyalty then

				-- if this is the player's pet unit frame, then announce the change in chat
				
				if unit_id == "pet" then
					
					if loyalty < 0 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: Warning... "..self.unit_name.." is "..(loyalty == -29 and "quickly" or "slowly").." losing loyalty ", 1, 0.5, 0.5 );
						
					elseif loyalty > 0 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: "..self.unit_name.." is "..(loyalty == 20 and "quickly" or "slowly").." gaining loyalty", 0.5, 1, 0.5 );
						
					elseif self.loyalty < 0 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: "..self.unit_name.." has stopped losing loyalty", 1, 0.83, 0 );
						
					elseif self.loyalty > 0 then
						
						DEFAULT_CHAT_FRAME:AddMessage( "nUI: "..self.unit_name.." has stopped gaining loyalty", 1, 0.83, 0 );
						
					end
				end
				
				self.loyalty = loyalty;
				
			end
		end		
	end
end

-------------------------------------------------------------------------------

function prototype:updateModel()

	local unit_id = self.layout.unit_id;
		
	if UnitExists( unit_id ) and self.frame.model then
	
		-- if we need to swap in the unit portrait...
		
		if self.frame.model.portrait == nil and (self.layout.Portrait.visible == "yes" or UnitIsVisible( unit_id ) == nil) then

			if self.frame.model.anim then
				self.Cache.anim:Hide();
				self.frame.model.anim = nil;
			end
	
			self.frame.model.portrait = self.Cache.portrait;
			self.frame.model.portrait:Show();

		-- if we need to swap in the unit animation...
		
		elseif self.frame.model.anim == nil and self.layout.Portrait.visible == "anim" and UnitIsVisible( unit_id ) then

			if self.frame.model.portrait then
				self.frame.model.portrait:Hide();
				self.frame.model.portrait = nil;
			end

			self.frame.model.anim = self.Cache.anim;
			self.frame.model.anim:Show();
		end

		-- refresh the animation or portrait as the case may be...
		
		if self.frame.model.anim then 
			self.frame.model.anim:SetUnit( unit_id );
			self.frame.model.anim:SetCamera( 0 );
		elseif self.frame.model.portrait then
			SetPortraitTexture( self.frame.model.portrait.texture, unit_id ); 
		end	
	end					
end	

-------------------------------------------------------------------------------

function prototype:checkGUID()

	local unit_id = self.layout.unit_id;
	local exists  = UnitExists( unit_id );
	local visible = self.frame:IsShown();

	-- see if the object this unit refers to changed
	
	if exists then
	
		-- this is a globally unique identifier for every toon and mob in the world
		
		local guid = UnitGUID( unit_id );
		
		-- if the identifier changed, then we need to redefine the unit accordingly
		
		if guid ~= self.guid then
			
			-- remember who we're talking about now
			
			self.guid      = guid;
			self.unit_name = UnitName( unit_id );
			
			-- update the unit content accordingly
						
			self:updateUnitClass();
			self:updatePowerBar();
			self:updateModel();
			self:updateUnitInfo();
			self:updateFaction();
			self:updateHappiness();
			self:updateResting();
			self:updateRaidTarget();
		end

		-- this addresses a quirk when zoning that sometimes party member names
		-- are not immediately available... note we are doing this whether the GUID
		-- changed or not. It catches us up if the name wasn't available previously
		
		self.unit_name = (UnitName( unit_id ) or "<unknown>");
		self.frame.label.text:SetText( self.unit_name );
				
	end
end

-------------------------------------------------------------------------------

function prototype:updateCombatStatus( arg2, arg3, arg4, arg5 )

	if self.CombatFeedback and UnitIsUnit( arg1, self.layout.unit_id ) then
		self.CombatFeedback( arg2, arg3, arg4, arg5 );
	end
end

-------------------------------------------------------------------------------

function prototype:updateFaction()

	local unit_id  = self.layout.unit_id;
	local pvp_icon = self.frame.pvp;
	local is_pvp   = UnitIsPVP( unit_id );
	local is_ffa   = UnitIsPVPFreeForAll( unit_id );
	local factionGroup, factionName = UnitFactionGroup( unit_id );

	-- update the unit's PVP status
	
	if is_ffa then
		
		if unit_id == "player" and not pvp_icon:IsShown() then
			
			PlaySound("igPVPUpdate");

			pvp_icon:SetScript( "OnEnter", 
				function()
					GameTooltip_AddNewbieTip( PVPFFA, 1.0, 1.0, 1.0, NEWBIE_TOOLTIP_PVPFFA, 1);
				end
			);
			pvp_icon:SetScript( "OnLeave", 
				function() 
					GameTooltip:Hide(); 
				end 
			);
		
		end
		
		pvp_icon.texture:SetTexture( "Interface\\TargetingFrame\\UI-PVP-FFA" );
		pvp_icon.texture:SetTexCoord( 0.03125, 0, 0.03125, 0.59375, 0.625, 0, 0.625, 0.59375 );
		pvp_icon:Show();

	elseif factionGroup and is_pvp then
		
		if unit_id == "player" and not pvp_icon:IsShown() then

			PlaySound("igPVPUpdate");
			PlayerPVPIconHitArea.tooltipTitle = factionName;
			PlayerPVPIconHitArea.tooltipText = getglobal("NEWBIE_TOOLTIP_"..strupper(factionGroup));
			PlayerPVPIconHitArea:Show();

			pvp_icon:SetScript( "OnEnter", 
				function()
					GameTooltip_AddNewbieTip( factionName, 1.0, 1.0, 1.0, getglobal( "NEWBIE_TOOLTIP_"..strupper(factionGroup) ), 1);
				end
			);
			pvp_icon:SetScript( "OnLeave", 
				function() 
					GameTooltip:Hide(); 
				end 
			);
					
		end
		
		pvp_icon.texture:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..factionGroup);
		pvp_icon.texture:SetTexCoord( 0.03125, 0, 0.03125, 0.59375, 0.625, 0, 0.625, 0.59375 );
		pvp_icon:Show();

	elseif pvp_icon:IsShown() then
		
		pvp_icon:Hide();
		pvp_icon:SetScript( "OnEnter", nil );
		pvp_icon:SetScript( "OnLeave", nil );

	end
	
	-- update the unit's faction
	
	local r, g, b, a = 0, 0, 0, 0.75;

	self.is_friend = true;

	-- if the unit is the player, the player's pet or a party or raid member, then
	-- there are only two possible states -- no reaction or colored green for PvP
	
	if UnitIsUnit( "player", unit_id ) or UnitIsUnit( "pet", unit_id ) or UnitPlayerOrPetInParty( unit_id ) == 1 or UnitPlayerOrPetInRaid( unit_id ) == 1 then
		
		if is_pvp or is_ffa then
			
			g = 0.5;
			
		end
		
	-- is this unit another player?
	
	elseif UnitPlayerControlled( unit_id ) then
			
		-- is the other player hostile to this player?
		
		if UnitCanAttack( unit_id, "player" ) then
			
			if UnitCanAttack( "player", unit_id ) then
				
				self.is_friend = false;
				r = 0.5;
			
			else
				
				b = 0.5;
				
			end
			
		-- could this player be hostile to the other player?
		
		elseif UnitCanAttack( "player", unit_id ) then
			
			r = 0.5;
			g = 0.5;
			self.is_friend = false;
		
		-- other players this player can assist are colored green
		
		elseif is_pvp and not UnitIsPVPSanctuary( "player" ) and not UnitIsPVPSanctuary( unit_id ) then
			
			g = 0.5;

		-- all other players are blue
		
		else
			
			b = 0.5;
			
		end

	-- could the player attack this unit?
	
	elseif UnitCanAttack( "player", unit_id ) then

		local reaction   = UnitReaction( "player", unit_id );
		local unit_level = UnitLevel( unit_id );
		
		self.if_friend = false;
		
		-- has someone else already tagged this mob?
		
		if UnitIsTapped( unit_id ) and not UnitIsTappedByPlayer( unit_id ) then
		
			r = 0.5;
			g = 0.5;
			b = 0.5;
			
		-- if it's hostile to the player, it's red
		
		elseif reaction >= 0 and reaction <= 3 then

			r = 0.5;
		
		-- otherwise it's yellow
		
		else				

			r = 0.5;
			g = 0.5;
			a = 0.90;
			
		end							
	
	
	-- paint friendly NPCs green
	
	elseif UnitIsFriend( "player", unit_id ) then

		g = 0.5;
			
	end

	-- paint the unit's label bar according to it's reaction to us
	
	if self.layout.show_reaction ~= "no" then
		self.frame.label:SetBackdropColor( r, g, b, a );
	end	
	
	-- update the unit level to ensure it is colored correctly
	
	self:updateUnitLevel();
	
end

-------------------------------------------------------------------------------

function prototype:updateRaidTarget()

	local unit_id = self.layout.unit_id;
	
	if UnitExists( unit_id ) then
		
		local idx = GetRaidTargetIndex( unit_id );

		if idx == nil then
			
			if self.frame.raid_icon:IsShown() then
				self.frame.raid_icon:Hide();
			end
			
		else
			
			local col = ((idx-1) % 4) * 0.25;
			local row = math.floor( (idx-1) / 4 ) * 0.25;
			
			self.frame.raid_icon.texture:SetTexCoord( col, row, col, row+0.25, col+0.25, row, col+0.25, row+0.25 );
			self.frame.raid_icon:Show();
			
		end
		
	elseif self.frame.raid_icon:IsShown() then
		
		self.frame.raid_icon:Hide();
		
	end
end

-------------------------------------------------------------------------------

function prototype:updateStatus()

	local unit_id = self.layout.unit_id;
	
	if UnitExists( unit_id ) and UnitIsUnit( unit_id, (arg1 or unit_id)) then

		-- update the unit's state -- AFK, OFFLINE, Feigning Death... etc
		
		self.frame.state:Hide();
	
		if not UnitIsConnected( unit_id ) then
			if not UnitIsUnit( "pet", unit_id ) then
				self.frame.state:Show();
				self.frame.state.text:SetText( "OFFLINE" );
			end
		elseif UnitIsAFK( unit_id ) then
			self.frame.state:Show();
			self.frame.state.text:SetText( "AFK" );
		elseif UnitIsFeignDeath( unit_id ) then
			self.frame.state:Show();
			self.frame.state.text:SetText( "FD" );
		elseif UnitIsDead( unit_id ) then
			self.frame.state:Show();
			self.frame.state.text:SetText( "DEAD" );
		elseif UnitIsDeadOrGhost( unit_id ) then
			self.frame.state:Show();
			self.frame.state.text:SetText( "GHOST" );
		end
	end			
end

-------------------------------------------------------------------------------

function prototype:enterCombat()

	if self.frame.model and not self.in_combat then

		local unit_id = self.layout.unit_id;
		
		if UnitIsUnit( unit_id, "player" ) or UnitIsUnit( unit_id, "pet" ) then
				
			local status   = self.frame.model.status;
			
			status.texture:SetTexCoord( 0.5, 0, 0.5, 1, 1, 0, 1, 1 );
			status:Show();
			status:SetAlpha( 1 );
			
			UIFrameFlash( status, 0.5, 0.5, 86400, true, 0, 0.75 );
			
			self.in_combat = true;
		end	
	end	
end

-------------------------------------------------------------------------------

function prototype:leaveCombat()
	
	self.in_combat = false;

	if self.frame.model then
			
		local status   = self.frame.model.status;
		
		if status:IsShown() then
			UIFrameFlashRemoveFrame( status );
			self.frame.model.status:Hide();
		end
	
		self:updateResting();
		
	end		
end

-------------------------------------------------------------------------------

function prototype:updateResting()
	
	if not self.in_combat and self.frame.model then
	
		local status = self.frame.model.status;

		if UnitIsUnit( "player", self.layout.unit_id ) and IsResting() then
			
			status.texture:SetTexCoord( 0, 0, 0, 1, 0.5, 0, 0.5, 1 );
			status:Show();
			status:SetAlpha( 1 );
			
			UIFrameFlash( status, 0.5, 0.5, 86400, true, 0, 0.75 );
			
		elseif status:IsShown() then
			
			UIFrameFlashRemoveFrame( status );
			status:Hide();		
			
		end		
	end	
end

-------------------------------------------------------------------------------
--        PERIODIC STATUS MANAGEMENT -- Called from events typically         --
-------------------------------------------------------------------------------

function prototype:refreshUnit()

	local frame   = self.frame;
	local unit_id = self.layout.unit_id;
	local exists  = UnitExists( unit_id );
	local visible = self.frame:IsShown();

	-- manage visibility

	if self.layout.is_visible ~= "no" then
		
		if exists and not visible then
			self:Show();
			visible = true;
			
		elseif not exists and visible then
			self:Hide();
			visible = false;
			
		end
	
	-- if this unit is to be force hidden, then hide it as expected
	
	elseif visible then
		
		self:Hide();
		visible = false;
		
	end
			
	-- manage changes in who the unit is pointing to if the unit is valid and active
	
	if exists and visible and self.parent:IsVisible() then
		
		-- check for changes in who the unit refers to
		
		self:checkGUID();
		
		-- collect all the transient status changes for this new unit			
		-- note... other methods depend on this being done first			
		
		self:updateStatus();
		
		-- this handles swapping animation in and out based on whether the unit in
		-- question is currently in or out of visibility range. Generally, WoW only
		-- keeps objects within 100 yards cached... if the unit you're looking at
		-- wanders outside of that range (roughly) their model animation stops
		-- working and you get a blank unit portrait. This test will swap the animation
		-- for a still portrait when they are out of range and back again when they
		-- return to visibility
		
		if frame.model then
			
			if self.layout.Portrait.visible == "anim" then
				
				if frame.model.anim ~= nil and UnitIsVisible( unit_id ) == nil then
					
					self:updateModel();
					
				elseif frame.model.anim == nil and UnitIsVisible( unit_id ) ~= nil then
					
					self:updateModel();
					
				end
			end
		end		
		
		-- this code checks to see if the unit is a party leader or not. Bliz's own code
		-- does not rely on the party leader change event and even notes that event to
		-- be buggy, so we follow their lead and check in the update routine.

		self:getRaidInfo();
		
		local raid_info = self.RaidInfo;
		
		if UnitIsPartyLeader( unit_id ) or (raid_info.in_raid and raid_info.rank == 2) then
			
			if not self.is_leader then
				
				self.frame.leader.texture:SetTexture( "Interface\\GroupFrame\\UI-Group-LeaderIcon.blp" );
				self.frame.leader.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
				self.frame.leader:Show();
				self.is_leader = true;
				self.is_assist = false;

			end
			
		elseif raid_info.in_raid and raid_info.rank == 1 then
			
			if not self.is_assist then
				
				self.frame.leader.texture:SetTexture( "Interface\\GroupFrame\\UI-GROUP-ASSISTANTICON.blp" );
				self.frame.leader.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
				self.frame.leader:Show();
				self.is_leader = false;
				self.is_assist = true;

			end
			
		elseif self.is_leader and self.frame.leader:IsShown() then
			
			self.frame.leader:Hide();
			self.is_leader = false;
			self.is_assist = false;
			
		end
		
		-- if we are in a raid, check to see if we are master looter, main tank and/or off-tank
		
		if raid_info.in_raid then
			
			if raid_info.is_ml and not self.frame.master_looter:IsShown() then
				self.frame.master_looter:Show();
			elseif self.frame.master_looter:IsShown() then
				self.frame.master_looter:Hide();
			end
			
			if not self.main_tank and raid_info.role == "maintank" then
				self.frame.tank.texture:SetTexture( "Interface\\GroupFrame\\UI-GROUP-MAINTANKICON.blp" );
				self.frame.tank.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
				self.frame.tank:Show();
				self.main_tank = true;
				self.off_tank  = false;
			elseif not self.off_tank and raid_info.role == "mainassist" then
				self.frame.tank.texture:SetTexture( "Interface\\GroupFrame\\UI-GROUP-MAINASSISTICON.blp" );
				self.frame.tank.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
				self.frame.tank:Show();
				self.main_tank = false;
				self.off_tank  = true;
			elseif self.main_tank or self.off_tank then
				self.frame.tank:Hide();
				self.main_tank = false;
				self.off_tank  = false;
			end
		end
	end
end

-------------------------------------------------------------------------------

function prototype:updateUnitInfo()

	-- display an icon if the unit is rare, elite or world boss if we're using portraits
	-- display info about the unit
	
	if UnitExists( self.layout.unit_id ) then
			
		if self.frame.model then
	
			local texture_file = nil;

			self.classification = UnitClassification( self.layout.unit_id );
		
			self.frame.model.info:Show();
			
			if self.classification == "rare" then
				self.frame.model.info.text:SetText( "Rare" );
				self.is_elite = false;
				texture_file  = "Interface\\AddOns\\nUI\\Units\\Art\\nUI_Icon_Rare";
			elseif self.classification == "rareelite" then
				self.frame.model.info.text:SetText( "Rare Elite" );
				self.is_elite = true;
				texture_file  = "Interface\\AddOns\\nUI\\Units\\Art\\nUI_Icon_Rare";
			elseif self.classification == "elite" then
				self.frame.model.info.text:SetText( "Elite" );
				self.is_elite = true;
				texture_file  = "Interface\\AddOns\\nUI\\Units\\Art\\nUI_Icon_Elite";
			elseif self.classification == "worldboss" then
				self.frame.model.info.text:SetText( "World Boss" );
				self.is_elite = true;
				texture_file  = "Interface\\AddOns\\nUI\\Units\\Art\\nUI_Icon_WorldBoss";
			else
				self.frame.model.info:Hide();
				self.is_elite = false;
				
				if self.classification ~= "normal" then
					nUI:debugMsg( 1, "unhandled unit classification ["..self.classification.."]" );
				end
			end
			
			if self.layout.elite_icon == "yes" then
				if texture_file ~= nil then
					
					texture_file = texture_file.."_"..self.layout.Portrait.style.."_"..self.layout.orientation..".blp";
					
					self.frame.model.elite.texture:SetTexture( texture_file );
					self.frame.model.elite.texture:ClearAllPoints();
					self.frame.model.elite.texture:SetAllPoints( self.frame.model.elite );
					self.frame.model.elite:Show();
				else
					self.frame.model.elite:Hide();
				end
			end
		end
	end
end

-------------------------------------------------------------------------------

function prototype:updateUnitClass()

	if UnitExists( self.layout.unit_id ) then		
			
		-- display the unit's class (warrior, rogue, shaman, etc.)
		
		self.localized_class, self.unit_class = UnitClass( self.layout.unit_id or "none" );

		local color = RAID_CLASS_COLORS[self.unit_class];

		if color and self.layout.class_colors ~= "no" then
			self.frame.label.text:SetTextColor( color.r, color.g, color.b, 1 );
		else
			self.frame.label.text:SetTextColor( 1, 0.8196079, 0, 1 );
		end
	
		if self.frame.unit_class then			
		
			local class_row, class_col;
			
			if self.unit_class == "WARRIOR" then
				class_row = 0;
				class_col = 0;
			elseif self.unit_class == "MAGE" then
				class_row = 0;
				class_col = 1;
			elseif self.unit_class == "ROGUE" then
				class_row = 0;
				class_col = 2;
			elseif self.unit_class == "DRUID" then
				class_row = 0;
				class_col = 3;
			elseif self.unit_class == "HUNTER" then
				class_row = 1;
				class_col = 0;
			elseif self.unit_class == "SHAMAN" then
				class_row = 1;
				class_col = 1;
			elseif self.unit_class == "PRIEST" then
				class_row = 1;
				class_col = 2;
			elseif self.unit_class == "WARLOCK" then
				class_row = 1;
				class_col = 3;
			elseif self.unit_class == "PALADIN" then
				class_row = 2;
				class_col = 0;
			else
				nUI:debugMsg( 1, "unhandled unit class ["..(self.unit_class or "unknown").."] in ["..self.layout.unit_id.."]" );
				class_row = 2;
				class_col = 1;
			end
		
			local x1 = class_col * 64;
			local x2 = x1 + 64;
			local y1 = class_row * 64;
			local y2 = y1 + 64;
			
			x1 = x1 / 256;
			x2 = x2 / 256;
			y1 = y1 / 256;
			y2 = y2 / 256;
			
			self.frame.unit_class.texture:SetTexCoord( x1, y1, x1, y2, x2, y1, x2, y2 );
		end
		
	end

end
